;
;	ZX Diagnostics - fixing ZX Spectrums in the 21st Century
;	https://github.com/brendanalford/zx-diagnostics
;
;	Original code by Dylan Smith
;	Modifications and 128K support by Brendan Alford
;
;	This code is free software; you can redistribute it and/or
;	modify it under the terms of the GNU Lesser General Public
;	License as published by the Free Software Foundation;
;	version 2.1 of the License.
;
;	This code is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;	Lesser General Public License for more details.
;
;	testram.asm
;	

;
;	Launcher for Spectrum Diagnostics Test Program
;
;	Performs a checksum on testrammain.bin, and launches
;	only if the checksum is as expected.
;
;	Checksum is generated by LUA scripting.
;
	LUA ALLPASS

	file = io.open("testrammain.bin", "rb")
	io.input(file)
	chk = 0
	local data = io.read(1)
	
	while data do
	    chk = (chk + string.byte(data)) % 0x10000
	    data = io.read(1)

	end
	io.close(file)
	sj.insert_define("TESTRAM_CHECKSUM", chk);

	ENDLUA

	org 24500
	
	di 
	ld ix, 24600
	ld hl, 0
	ld d, 0

;
;	Compute 16 bit addition checksum on code from
;	24600 to 32767
;

checksum
	
	ld a, (ix)
	ld e, a
	add hl, de
	inc ix
	ld a, ixh
	cp 0x80
	jr nz, checksum
	
	ld de, TESTRAM_CHECKSUM

	ld a, d
	cp h
	jr nz, error
	ld a, e
	cp l
	jr nz, error
	jp start
	
error
	ld l, 3
	ei
	jp 0x0055
	
	BLOCK 24600-$, #FF
	
start
	incbin "testrammain.bin"
